package org.one.stone.soup.mjdb.data.field;

/*
 * MJDB (Micro Java Database) Version 1.0
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.wet-wired.com/licence/license-os.html
 */

/**
	* The BoolField class is a wrapper for an single boolean value.
	* @author Nik Cross
*/

public class BoolField extends Field{
	public static String DEFAULT_ID = "_Bool";

	private boolean data = false;
/**
 * This constructor creates a new BoolField with a value of false.
 */

public BoolField() {
	super();
}
/**
 * This constructor sets the initial boolean value.
 * @param data	The boolean value
 */

public BoolField(boolean data) {
	super();

	this.data=data;
}
/**
 * Compares this object with the target object.
 * Returns EQ if the two boolean values are the same.
 * Returns GT if the target value is true and this objects value is false.
 * Returns LT if the target value is false and this objects value is true.
 * @param target	The Field to compare with
 */

public int compare(Field target) throws CompareException {

	if(target instanceof BoolField)
	{
		if( ((BoolField)target).getValue() == this.data )
			return EQ;
		else if( ((BoolField)target).getValue()==true && this.data==false )
			return GT;
		else
			return LT;
	}

	throw new CompareException("Can't compare "+data+" with boolean");
}
/**
 * Returns the boolean data held in this field.
 */
public boolean getValue() {
	return data;
}
/**
 * Returns the boolean data held in this field as a String.
 * @param name	The name of the data to return (Ignored in BoolField)
 */
public String getValue(String name) {
	return ""+data;
}
/**
 * BoolFields can't be compared.
 */
public boolean inRange(Object min, boolean inclusiveMin, Object max, boolean inclusiveMax) throws CompareException {
	throw new CompareException("Can't check range of boolean");
}
/**
 * Sets the boolean value held in this field.
 * @param name		The value to be changed (Ignored in BoolField)
 * @param value		The requested new value
 * @param source	The object requesting this change
 */

public void setValue(String name, String value, Object source) throws LockException{

	if(value.equals("true"))
		setValue(true,source);
	else
		setValue(false,source);
}
/**
 * Sets the boolean value held in this field.
 * @param newData	The requested new value
 * @param source	The object requesting this change
 */

public void setValue(boolean newData,Object source) throws LockException {

	if(locked && !isLockOwner(source))
		throw new LockException(this+"is locked");

	if(newData==data)
		return;

	data = newData;
	setChanged();

	if(report)
		reportChange(source);
}
public String toString() {
	return ""+data;
}
}
