package org.one.stone.soup.mjdb.data.functions;

import org.one.stone.soup.mjdb.data.field.*;
import org.one.stone.soup.mjdb.data.group.*;

/*
 * MJDB (Micro Java Database) Version 1.0
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.wet-wired.com/licence/license-os.html
 */

public final class Sort {
	public static final int ASCENDING = 1;
	public static final int DESCENDING = -1;

public static final void applyIndex(Table table, int[] index)throws SortException,SetException,LockException {

	//System.out.println("applyIndex");

	int size = table.getNoOfRows();
	if(index.length!=size)
		throw(new SortException());

	Table tempTable = new Table(table.getNoOfColumns());
	int loop;
	for(loop=0;loop<size;loop++)
		tempTable.addRow(table.getRow(loop),table);

	for(loop=0;loop<size;loop++)
		table.setRow(tempTable.getRow(index[loop]),loop,null);

	//System.out.println("applyIndex done");
}
public final static int[] createIndex(Table table,int column,int direction) throws SortException,DatabaseException {

	//System.out.println("createIndex");

	int size=table.getNoOfRows();

	if(size==0)
		throw(new SortException("Table has 0 rows"));

	if(direction!=ASCENDING && direction!=DESCENDING)
		throw(new SortException());

	int[] index = new int[size];
	boolean[] indexed = new boolean[size];

	int loop;
	for(loop=0;loop<size;loop++)
		indexed[loop]=false;

	int loop2;
	int foundIndex;
	Field foundObj;
	for(loop=0;loop<size;loop++)
	{
		//System.out.println("createIndex:"+loop);

		foundIndex=0;
		foundObj = null;

		for(loop2=0;loop2<size;loop2++)
		{
			//System.out.println("createIndex2:"+loop);

			if(indexed[loop2]==true)
				continue;

			if(foundObj==null)
			{
				foundObj=(Field)table.getField(column,loop2);
				foundIndex=loop2;
				continue;
			}

			try{
				if(direction==ASCENDING)
				{
					if(foundObj.compare(table.getField(column,loop2))==Field.GT)
					{
						foundObj=(Field)table.getField(column,loop2);
						foundIndex=loop2;
					}
				}
				else
				{
					if(foundObj.compare(table.getField(column,loop2))==Field.LT)
					{
						foundObj=(Field)table.getField(column,loop2);
						foundIndex=loop2;
					}
				}
			}
			catch(CompareException e)
			{
				throw(new SortException("Comparison exception "+foundObj+" & "+table.getField(column,loop2)));
			}
		}
//		System.out.println("Indexed "+loop+" -> "+foundIndex+" = "+(String)data.elementAt(foundIndex));
		indexed[foundIndex]=true;
		index[loop]=foundIndex;
	}
//	System.out.println("Index created");

	//System.out.println("createIndex done");

	return index;
}
public static final void sort(Table table,int column)throws SortException {
	sort(table,column,ASCENDING);
}
public static final void sort(Table table,int column,int direction)throws SortException {

	try{
		applyIndex(table,createIndex(table,column,direction));
	}
	catch(Exception e)
	{
		throw new SortException("sort: "+e);
	}
}
public static final void sort(Table table,String columnName)throws SortException {
	sort(table,columnName,ASCENDING);
}
public static final void sort(Table table,String columnName,int direction)throws SortException {

	int loopC=0;

	for(loopC=0;loopC<table.getNoOfColumns();loopC++)
	{
		if(table.getColumnTitle(loopC) instanceof String)
		{
			System.out.println("Found col:"+((String)table.getColumnTitle(loopC))+" =? "+columnName);
			if(((String)table.getColumnTitle(loopC)).equals(columnName))
			{
				System.out.println("Col match:"+loopC);
				break;
			}
		}
	}

	System.out.println("sort col:"+loopC);
	sort(table,loopC,direction);
}
}
