package org.one.stone.soup.mjdb.data.group;

import java.net.*;
import java.io.*;

import org.one.stone.soup.mjdb.data.field.*;
import org.one.stone.soup.mjdb.data.persistence.*;
import org.one.stone.soup.mjdb.data.functions.*;

/*
 * MJDB (Micro Java Database) Version 1.0
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.wet-wired.com/licence/license-os.html
 */

public class Database extends Table implements DataGroupInterface,FieldListener{

	private long MAX_LOCK_WAIT_TIME=1000;

	public static final int TABLE_NAME = 0;
	public static final int TABLE_FILE = 1;
	public static final int TABLE = 2;

	private String path;
	private String file;
	private char separator = ',';

	private boolean databaseOK = true;

	private Table lockTable;

private Database() throws DatabaseException{
	super();

	try{
		lockTable = new Table( 2 );
		lockTable.setColumnTitle(0,"locked_table",this);
		lockTable.setColumnTitle(1,"lock_owner",this);
	} catch(Exception e)
	{
		throw new DatabaseException( "Failed to create lock table" );
	}
}
public Database(String path,String fileName,char separator) throws DatabaseException{
	super();

	try{
		//path = System.getProperty("user.dir")+"\\"+path;

		loadData(path,fileName,separator);

		this.path = path;
		this.file = fileName;
		this.separator = separator;
	}
	catch(LockException e){}//Dummy exception
	catch(SetException e){
		e.printStackTrace();
	}
}
public Database(URL url,char separator) throws DatabaseException{
	super();

	this.separator = separator;
	try{
		loadData(url,separator);
	}
	catch(LockException e){}//Dummy exception
	catch(SetException e){
		e.printStackTrace();
	}
}
public Row addTable(String tableName,String tableFile, Table newTable) throws LockException,SetException{

	Row row = new Row(getNoOfColumns());
	row.setField(TABLE_NAME,tableName,this);
	row.setField(TABLE_FILE,tableFile,this);
	row.setField(TABLE,newTable,this);

	addRow(row,this);

	return row;
}
public String getFileName() {
	return file;
}
public static Database getNewDatabase(String path,String fileName) throws DatabaseException {

	Database newDb = new Database();

	try{
		newDb.setPath(path);
		newDb.setFileName(fileName);
		newDb.addColumn("table_name",newDb);
		newDb.addColumn("table_file",newDb);
		newDb.addColumn("table",newDb);

		newDb.loadData(path,fileName,newDb.separator);

		newDb.setName( "Database" );
		for(int loop=0;loop<newDb.getNoOfRows();loop++)
		{
			Table table = (Table)newDb.getField("table",loop);
			table.setName( ""+newDb.getField("table_name",loop) );
		}
	}
	catch(Exception e)
	{
		if(e instanceof DatabaseException)
			throw (DatabaseException)e;

		e.printStackTrace();
		return null;
	}

	return newDb;
}
public int getNoOfTables() {
	return getNoOfRows();
}
/**
 *
 * @return java.lang.String
 */
public String getPath() {
	return path;
}
/**
 *
 * @return char
 */
public char getSeparator() {
	return separator;
}
public Table getTable(int index) throws DatabaseException {
	try{
		return (Table)getField(TABLE,index);
	}
	catch(DatabaseException de)
	{
		throw new DatabaseException(de.getMessage()+" in table for index "+index);
	}
}
public Table getTable(String tableName) throws DatabaseException
{

	try{
	for(int loop=0;loop<getNoOfRows();loop++)
	{
		if( ((StringField)getField(TABLE_NAME,loop)).getValue().equals(tableName))
		{
			return (Table)getField(TABLE,loop);
		}
	}
	}
	catch(DatabaseException de)
	{
		throw new DatabaseException(de.getMessage()+" in database table list");
	}
	catch(ClassCastException ce)
	{
		throw new DatabaseException("Failed to find table "+tableName);
	}

	throw new DatabaseException("Table "+tableName+" not found in table list");
}
public Object getTableField(int tableIndex,int colIndex,int rowIndex) throws DatabaseException{

	Table table = null;

	try{
		table = (Table)getField(TABLE,tableIndex);
	}
	catch(DatabaseException de)
	{
		throw de;
	}

	try{
		return table.getField(colIndex,rowIndex);
	}
	catch(DatabaseException de)
	{
		throw new DatabaseException(de.getMessage()+" in table for index "+tableIndex);
	}
}
public int getTableIndex(String tableName) throws DatabaseException {

	try{
		for(int loop=0;loop<getNoOfRows();loop++)
		{
			if( ((StringField)getField(TABLE_NAME,loop)).getValue().equals(tableName))
			{
				return loop;
			}
		}
	}
	catch(DatabaseException de)
	{
		throw de;
	}

	throw new DatabaseException("Table "+tableName+" not found in database");
}
public int getTableNoOfCols(int index) throws DatabaseException{
	return ((Table)getField(TABLE,index)).getNoOfColumns();
}
public int getTableNoOfRows(int index) throws DatabaseException{
	return ((Table)getField(TABLE,index)).getNoOfRows();
}
public Row getTableRow(int tableIndex,int rowIndex) throws DatabaseException{

	Table table = null;
	try{
		table = (Table)getField(TABLE,tableIndex);
	}
	catch(DatabaseException de)
	{
		throw de;
	}

	return table.getRow(rowIndex);
}
/**
 * getValue method comment.
 */
public String getValue(String name) {
	return null;
}
/**
 *
 * @return boolean
 */
public boolean isDatabaseOK() {
	return databaseOK;
}
public Table join(String tableNameA, String columnNameA, String tableNameB, String columnNameB) throws DatabaseException {

	try{

	System.out.println("join "+tableNameA+"."+columnNameA+" to "+tableNameB+"."+columnNameB);

		Table tableA=(Table)getField(1,getTableIndex(tableNameA));
		int columnA=tableA.getColumnTitleIndex(columnNameA);
		Table tableB=(Table)getField(1,getTableIndex(tableNameB));
		int columnB=tableB.getColumnTitleIndex(columnNameB);

	System.out.println("join "+tableA+"."+columnA+" to "+tableB+"."+columnB);

		Table result = Join.join(tableA,columnA,tableB,columnB);

		System.out.println("Join done");

		return result;
	}
	catch(Exception e)
	{
		throw new DatabaseException(e.getMessage());
	}
}
public void loadData(String path,String fileName,char separator) throws LockException,SetException,DatabaseException{

	InputStream iStream;
	CsvToTable csv;

	try{
		File file = new File(path+fileName);
		iStream = new FileInputStream( ""+file.getAbsoluteFile() );
		csv = new CsvToTable((Table)this,iStream,separator,true);
		if(getNoOfColumns()<3)
		{
				addColumn("table",this);
		}

		//tables = csv.getTable();
	}
	catch(Exception e)
	{
		setDatabaseOK(false);

		File file = new File(path+fileName);

		throw new DatabaseException("Error loading definition table: "+e.getMessage()+" Full file path:"+file.getAbsoluteFile());
	}
	autoWrap(this);
	System.out.println("Loaded definition table: "+path+fileName);

	int numberOfTables = getNoOfRows();
	String tableName;

	int loop=0;
	for(loop=0;loop<numberOfTables;loop++)
	{
		tableName = ((StringField)getField(TABLE_FILE,loop)).getValue();
		try{
			File file = new File(path+tableName);
			iStream = new FileInputStream( ""+file.getAbsolutePath() );
			csv = new CsvToTable(iStream,separator);
			setField(TABLE,loop,csv.getTable(false),this);
		}
		catch(FileNotFoundException e)
		{
			setDatabaseOK(false);
			throw new DatabaseException("Error loading table "+tableName+": "+e);
		}
		catch(PersistenceException e)
		{
			setDatabaseOK(false);
			throw new DatabaseException("Error loading table "+tableName+": "+e);
		}
		catch(DatabaseException e)
		{
			setDatabaseOK(false);
			throw new DatabaseException("Error loading table "+tableName+": "+e);
		}
		System.out.println("Loaded table: "+path+tableName);
	}
	System.out.println("Loaded "+loop+" tables");
	if(loop==0)
	{
		setDatabaseOK(false);
		System.out.println("No tables loaded");
	}
}
public void loadData(URL url,char separator) throws LockException,SetException,DatabaseException{

	InputStream iStream;
	CsvToTable csv;

	try{
		iStream = url.openStream();
		csv = new CsvToTable((Table)this,iStream,separator,true);
		if(getNoOfColumns()<3)
		{
				addColumn("table",this);
		}

		//tables = csv.getTable();
	}
	catch(Exception e)
	{
		setDatabaseOK(false);
		throw new DatabaseException("Error loading definition table: "+e.getMessage());
	}
	autoWrap(this);
	System.out.println("Loaded definition table: "+url);

	int numberOfTables = getNoOfRows();
	String tableName;

	int loop=0;
	for(loop=0;loop<numberOfTables;loop++)
	{
		tableName = ((StringField)getField(TABLE_FILE,loop)).getValue();
		try{
			iStream = new URL(tableName).openStream();
			csv = new CsvToTable(iStream,separator);
			setField(TABLE,loop,csv.getTable(false),this);
		}
		catch(MalformedURLException e)
		{
			setDatabaseOK(false);
			throw new DatabaseException("Error loading table "+tableName+": "+e);
		}
		catch(IOException e)
		{
			setDatabaseOK(false);
			throw new DatabaseException("Error loading table "+tableName+": "+e);
		}
		catch(PersistenceException e)
		{
			setDatabaseOK(false);
			throw new DatabaseException("Error loading table "+tableName+": "+e);
		}
		catch(DatabaseException e)
		{
			setDatabaseOK(false);
			throw new DatabaseException("Error loading table "+tableName+": "+e);
		}
		System.out.println("Loaded table: "+tableName);
	}
	System.out.println("Loaded "+loop+" tables");
	if(loop==0)
	{
		setDatabaseOK(false);
		System.out.println("No tables loaded");
	}
}
public LockKey obtainTableLock( String tableName,Object requester ) throws LockException,DatabaseException
{

	Table target = getTable(tableName);

	LockKey key = new LockKey( target,requester );

	long startTime = System.currentTimeMillis();

	while( (System.currentTimeMillis()-startTime)<MAX_LOCK_WAIT_TIME )
	{
		if( !target.isLocked() )
		{
			target.lock( key );
			lockTable.addRow( key,requester );
			break;
		}
	}

	return key;
}
public void releaseTableLock( LockKey key ) throws LockException,DatabaseException
{
	Table target = (Table)key.getField(0);

	saveTable( target );

	target.unlock( key );

	lockTable.removeRow( key,key.getLockOwner() );
}
public boolean saveData() throws DatabaseException
{
	if(isDatabaseOK()==false)
	{
		throw new DatabaseException("Database in error. Database not saved.");
	}

	reportChange(this);

	OutputStream out=null;
 try{

	for(int loopT=0;loopT<getNoOfTables();loopT++)
	{
		out = new FileOutputStream( path+getField(TABLE_FILE,loopT).toString() );
		new TableToCsv(out,separator,getTable(loopT));
		out.close();
	}

/*	try{
		if(getNoOfColumns()>=3)
			removeColumn(TABLE);
	}
	catch(LockException le){}*/

	out = new FileOutputStream( path+file );
	new TableToCsv(out,separator,(Table)this);
	out.close();
 }
 catch(IOException ioe)
 {
	 ioe.printStackTrace();
	 try{
		 out.close();
	 }
	 catch(IOException ie){}
	 return false;
 }
 return true;
}
public boolean saveTable( String tableName ) throws DatabaseException
{
	return saveTable( getTable(tableName) );
}
public boolean saveTable( Table table ) throws DatabaseException
{
	if(isDatabaseOK()==false)
	{
		throw new DatabaseException("Database in error. Table not saved.");
	}

	OutputStream out=null;
 try{

	reportChange(this);

	for(int loopT=0;loopT<getNoOfTables();loopT++)
	{
		if( getTable(loopT)==table )
		{
			File file = new File(path+getField(TABLE_FILE,loopT).toString());
			out = new FileOutputStream( ""+file.getAbsoluteFile() );
			new TableToCsv(out,separator,getTable(loopT));
			out.close();

			//System.out.println( "Saved "+ path+getField(TABLE_FILE,loopT).toString() );
			break;
		}
	}
 }
 catch(IOException ioe)
 {
	 ioe.printStackTrace();
	 try{
		 out.close();
	 }
	 catch(IOException ie){}
	 return false;
 }
 return true;
}
public Table search(String tableName, int columnIndex, String data) {

	try{
		return Search.search( (Table)getField(TABLE,getTableIndex(tableName)) , columnIndex , data );
	}
	catch(Exception e)
	{
		System.out.println("Exception in search: "+e);
		return null;
	}
}
public Table search(String tableName, String data) {

	try{
		return Search.search( (Table)getField(TABLE,getTableIndex(tableName)) , data );
	}
	catch(Exception e)
	{
		System.out.println("Exception in search: "+e);
		return null;
	}
}
public Table search(String tableName, String columnName, String data) {

	try{
		return Search.search( (Table)getField(TABLE,getTableIndex(tableName)) , columnName , new StringField(data) );
	}
	catch(Exception e)
	{
		System.out.println("Exception in search: "+e);
		return null;
	}
}
public Table search(String tableName, String columnName, String min,String max) {

	try{
		return Search.search( (Table)getField(TABLE,getTableIndex(tableName)) , columnName , min,max );
	}
	catch(Exception e)
	{
		System.out.println("Exception in search: "+e);
		return null;
	}
}
public Table search(Table table, int columnIndex, String data) {

	try{
		return Search.search( table , columnIndex , data );
	}
	catch(Exception e)
	{
		System.out.println("Exception in search: "+e);
		return null;
	}
}
/**
 *
 * @param newDatabaseOK boolean
 */
private void setDatabaseOK(boolean newDatabaseOK) {
	databaseOK = newDatabaseOK;
}
public void setFileName(String newFile) {
	file = newFile;
}
/**
 *
 * @param newPath java.lang.String
 */
public void setPath(String newPath) {
	path = newPath;
}
/**
 *
 * @param newSeparator char
 */
public void setSeparator(char newSeparator) {
	separator = newSeparator;
}
public void sort(String tableName, String data) {

	try{
		Sort.sort( (Table)getField(TABLE,getTableIndex(tableName)) , data );
		System.out.println("Sort done");
	}
	catch(Exception e)
	{
		System.out.println("Exception in sort: "+e);
	}
}
public void sort(String tableName, String data, int direction) {

	try{
		Sort.sort( (Table)getField(TABLE,getTableIndex(tableName)) , data , direction);
		System.out.println("Sort done");
	}
	catch(Exception e)
	{
		System.out.println("Exception in sort: "+e);
	}
}

public void autoWrapAll(Object source) throws LockException,SetException,DatabaseException{

	for(int loop=0;loop<getNoOfTables();loop++)
	{
		getTable(loop).autoWrap(source);
	}

	super.autoWrap(source);
}

public Table waitTableUnlock( String tableName ) throws LockException,DatabaseException
{

	Table target = getTable(tableName);

	long startTime = System.currentTimeMillis();

	while( (System.currentTimeMillis()-startTime)<MAX_LOCK_WAIT_TIME )
	{
		if( !target.isLocked() )
		{
			return target;
		}
	}

	throw new LockException("Table "+tableName+" failed to unlock in time. Lock owner:"+target.getLockOwner());
}
public void addListener(FieldListener listener) {
	super.addListener( listener );

	for(int loop=0;loop<getNoOfRows();loop++)
	{
		getRow(loop).addListener(this);
		((Table)getRow(loop).getField(TABLE)).addListener(this);
	}
}
	/* (non-Javadoc)
	 * @see org.one.stone.soup.mjdb.data.field.FieldListener#receiveChange(java.lang.Object, java.lang.Object)
	 */
	public void receiveChange(Object source, Object data) {
		reportChange(source);
	}
}
