package org.one.stone.soup.server.http.client;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.Socket;
import java.net.SocketAddress;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.nio.channels.SocketChannel;

import org.one.stone.soup.io.Connection;
import org.one.stone.soup.io.ConnectionListener;
import org.one.stone.soup.io.StreamConnection;
import org.one.stone.soup.stringhelper.StringGenerator;

public class HttpTunnelConnection implements Connection,ConnectionListener
{
	private String alias;
	private String address;
	private int port;
	private String url;
	private String host;
	
	private boolean useProxy = false;
	
	private InputStream iStream;
	private OutputStream oStream;
	
	private Socket upSocket;
	private Socket downSocket;
	
	public HttpTunnelConnection(String host,String tunnelToAddress,int tunnelToPort) throws UnknownHostException,IOException
	{
		this(host,tunnelToAddress,tunnelToPort,StringGenerator.generateUniqueId());
	}
	
	public HttpTunnelConnection(String host,String tunnelToAddress,int tunnelToPort,String id) throws UnknownHostException,IOException
	{
		this.host = host;
		this.address = host;
		this.port=80;
		
		alias = InetAddress.getLocalHost().getHostAddress()+"-"+id;
		
		url = "/tunnel/"+tunnelToAddress+"/"+tunnelToPort+"/"+alias;
		
		setProxy();
		
		openUpConnection();
		openDownConnection();
	}
	
	private void setProxy()
	{
		String proxySet = System.getProperty("proxySet");
		if(proxySet!=null && proxySet.equals("true"))
		{
			useProxy = true;
			address = System.getProperty("proxyHost");
			port = Integer.parseInt(System.getProperty("proxyPort"));
		}
	}
	
	private void openUpConnection() throws IOException
	{
		upSocket = new Socket(address,port);
		
		//GET http://www.onestonesoup.org/ProxyWatch.html HTTP/1.1
		//Host: www.onestonesoup.org
		//User-Agent:
				
		OutputStream upStream = upSocket.getOutputStream();
		upStream.write( ("GET "+url+" HTTP/1.1\r\n").getBytes() );
		upStream.write( ("Host: "+host+"\r\n").getBytes() );
		upStream.write( ("User-Agent: Tunnel Client\r\n\r\n").getBytes() );
		upStream.flush();
		
		iStream = upSocket.getInputStream();
	}
	private void openDownConnection() throws IOException
	{
		downSocket = new Socket(address,port);

		OutputStream upStream = downSocket.getOutputStream();
		upStream.write( ("POST "+url+" HTTP/1.1\r\n").getBytes() );
		upStream.write( ("Host: "+host+"\r\n").getBytes() );
		upStream.write( ("User-Agent: Tunnel Client\r\n\r\n").getBytes() );
		upStream.flush();
		
		oStream = downSocket.getOutputStream();
	}

	public OutputStream getOutputStream()
	{
		return oStream;
	}
	
	public InputStream getInputStream()
	{
		return iStream;
	}
	
	public void close() throws IOException {
		upSocket.close();
		downSocket.close();
	}

	public String getAlias() {
		return alias;
	}

	public long getDataSentSize() {
		return -1;
	}

	public void connectionClosed(Connection owner) {
		try{
			close();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}