package org.one.stone.soup.util;

import java.util.*;
import java.awt.*;
import java.awt.event.*;

/*
 * Wet-Wired.com Library Version 2.1
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.onestonesoup.org/OSSLicense.html
 */

/**
 * @author nikcross
 *
 */

// TODO
public class ClockWatch implements Runnable{

	public static final int TEN_SECOND = 1;
	public static final int HOURLY = 2;
	public static final int DAILY = 3;
	public static final int WEEKLY = 4;
	public static final int MONTHLY = 5;
	public static final int YEARLY = 6;

	private int waitType=HOURLY;
	private String actionCommand="ready";
	private Object source;

	private ActionListener actionListener;

	private boolean running=false;

	private int lastRun;

	private long sleepTime = 10000;
public ClockWatch(int waitType,String actionCommand) {
	this(waitType,actionCommand,null);
}
public ClockWatch(int waitType,String actionCommand,Object source) {
	super();

	this.waitType=waitType;
	this.actionCommand = actionCommand;
	this.source = source;

	// Create check times
	switch( waitType )
	{
		case TEN_SECOND:
			sleepTime = 1000; // One second
			break;
		case HOURLY:
			sleepTime = 30000; // Half minute
			break;
		case DAILY:
			sleepTime = 3600000; // Hourly
			break;
		case WEEKLY:
			sleepTime = 86400000; // Daily
			break;
		case MONTHLY:
			sleepTime = 86400000; // Daily
			break;
		case YEARLY:
			sleepTime = 86400000; // Daily
			break;
	}
}
	public synchronized void addActionListener(ActionListener l) {
		actionListener = AWTEventMulticaster.add(actionListener, l);
	}
/**
 *
 * @return java.lang.String
 */
public String getActionCommand() {
	return actionCommand;
}
/**
 *
 * @return long
 */
public long getWaitType() {
	return waitType;
}
private void processActionEvent(ActionEvent e) {
	if (actionListener != null) {
		actionListener.actionPerformed(e);
	}
}
	public synchronized void removeActionListener(ActionListener l) {
		actionListener = AWTEventMulticaster.remove(actionListener, l);
	}
public void run()
{
	if(running)
		return;

	running=true;

	while(running)
	{
		try{Thread.sleep(sleepTime);}catch(Exception e){}//dummy catch

		boolean act = false;
		Calendar cal = Calendar.getInstance();

		switch( waitType )
		{
			case TEN_SECOND:
				if((cal.get(Calendar.SECOND)/10)!=lastRun)
				{
					lastRun = cal.get(Calendar.SECOND)/10;
					act=true;
				}
				break;
			case HOURLY:
				if(cal.get(Calendar.HOUR_OF_DAY)!=lastRun)
				{
					lastRun = cal.get(Calendar.HOUR_OF_DAY);
					act=true;
				}
				break;
			case DAILY:
				if(cal.get(Calendar.DAY_OF_YEAR)!=lastRun)
				{
					lastRun = cal.get(Calendar.DAY_OF_YEAR);
					act=true;
				}
				break;
			case WEEKLY:
				if(cal.get(Calendar.WEEK_OF_YEAR)!=lastRun)
				{
					lastRun = cal.get(Calendar.WEEK_OF_YEAR);
					act=true;
				}
				break;
			case MONTHLY:
				if(cal.get(Calendar.MONTH)!=lastRun)
				{
					lastRun = cal.get(Calendar.MONTH);
					act=true;
				}
				break;
			case YEARLY:
				if(cal.get(Calendar.YEAR)!=lastRun)
				{
					lastRun = cal.get(Calendar.YEAR);
					act=true;
				}
				break;
		}

		if(act==true)
		{
			if(source==null)
				processActionEvent(new ActionEvent(this,waitType,actionCommand,lastRun));
			else
				processActionEvent(new ActionEvent(source,waitType,actionCommand,lastRun));
		}
	}

	running=false;
}
/**
 *
 * @param newActionCommand java.lang.String
 */
public void setActionCommand(String newActionCommand) {
	actionCommand = newActionCommand;
}
public void start()
{
	Thread thread = new Thread(this,"ClockWatch ["+waitType+"]:"+actionCommand);
	thread.start();
}
public void stop()
{
	running=false;
}
}
