package org.one.stone.soup.xapp.controller.xml;

import java.io.File;

import javax.swing.JTextArea;

import org.one.stone.soup.xapp.XappComponentStore;
import org.one.stone.soup.xapp.XappRootApplication;
import org.one.stone.soup.xapp.components.XappComponent;
import org.one.stone.soup.xapp.components.form.XForm;
import org.one.stone.soup.xapp.components.tree.XTree;
import org.one.stone.soup.xapp.containers.XappPopup;
import org.one.stone.soup.xapp.xpath.XappXPathDB;
import org.one.stone.soup.xapp.xpath.XappXPathDBChangeListener;
import org.one.stone.soup.xapp.xpath.XappXPathDB.XappSubXPathDB;
import org.one.stone.soup.xml.XmlElement;

public class XmlApplicationController implements XappXPathDBChangeListener{	
	private XmlElement currentContext;
	
	//private XmlElement modelRoot;
	private XmlSchemaController schema;
	private XmlSchemaViewController viewController;
	
	private XappSubXPathDB viewDB;
	
	//private XmlPopupFactory popupFactory;
	
	//private XmlApplication application;
	private XappPopup popup;
	private XTree tree;
	private XForm form;
	
	public XmlApplicationController(File root,XappSubXPathDB xSchemaDB,XmlElement xModelRoot,XappSubXPathDB xViewDB,XmlApplication application)
	{
		//this.application = application;
		viewDB = xViewDB;
		viewDB.addListener(this);
		viewController = new XmlSchemaViewController(root,viewDB.getRoot());
		
		xSchemaDB.addListener(this);
		schema = new XmlSchemaController(xSchemaDB);
		//popupFactory = new XmlPopupFactory(this,schema);
		
		//modelRoot = xModelRoot;
		if(xModelRoot==null)
		{
			xModelRoot = schema.createRoot();
		}
		
		currentContext = xModelRoot;
		setModelRoot(xModelRoot);
	}
	
	public void updateNode(XmlElement data)
	{
		boolean hasChanged = false;
		
		for(int loop=0;loop<data.getElementCount();loop++)
		{
			XmlElement field = data.getElementByIndex(loop);
			String name = field.getName();
			String value = field.getValue().trim();
			
			if(name.equals("@value"))
			{
				String oldValue = currentContext.getValue();
				if(!oldValue.equals(value))
				{
					hasChanged=true;
				}
				
				if(value.length()>0)
				{
					currentContext.setValue(value,0);
				}
				else
				{
					if(currentContext.getElementCount()>0 && currentContext.getElementByIndex(0)==null)
					{
						currentContext.removeData(0);
					}
				}
			}
			else if(currentContext.getAttributeValueByName(name)!=null && !currentContext.getAttributeValueByName(name).equals(value))
			{
				hasChanged=true;
				currentContext.setAttributeValue(name,value);
			}
		}
		
		if(hasChanged)
		{
			tree.refreshView();
			if(XmlApplication.autoSaveIsOn())
			{
				XmlApplication.save();
			}
		}
	}
	
	public void setModelRoot(XmlElement modelRoot)
	{
		//this.modelRoot = modelRoot;
		if(tree==null)
		{
			tree = XappRootApplication.getComponentFactory().buildXTree(modelRoot,viewController);
		}
		tree.addReceiver(XappRootApplication.getRoot());
		tree.buildTree(modelRoot);
		tree.refreshView();
	}
	
	public void setSchema(File root,XappSubXPathDB xSchemaDB,XappSubXPathDB xSchemaViewDB,boolean recreateDocument)
	{
		viewDB = xSchemaViewDB;
		viewDB.addListener(this);
		
		viewController = new XmlSchemaViewController(root,viewDB.getRoot());
		
		tree.setViewController(viewController);
		
		xSchemaDB.addListener(this);
		schema = new XmlSchemaController(xSchemaDB);
		
		if(recreateDocument==true)
		{
			setModelRoot( schema.createRoot() );
		}
	}
	
	public XmlElement getModelRoot()
	{
		return tree.getData();
	}
	
	public void setContext(XmlElement context)
	{
		if(form!=null)
		{
			updateNode( form.getData() );
		}
		
		XmlElement xDefinition = schema.getDefinitionFor(context.getName());
		if(xDefinition==null)
		{
			return;
		}
		XmlElement xNotes = xDefinition.getElementByName("notes");
		XappComponent notes = XappRootApplication.getComponent("notes");
		if(xNotes!=null && notes!=null && notes instanceof XForm)
		{
			((JTextArea)((XForm)notes).getElement("notes")).setText( xNotes.getValue() );
		}
		
		currentContext = context;
	}
	
	public XmlElement getContext()
	{
		return currentContext;
	}
	
	public XTree getTreeView()
	{
		return tree;
	}
	
	public XappPopup getPopup()
	{
		if(popup!=null)
		{
			popup.hide();
		}
		
		popup = new XmlPopupFactory(this,schema).createPopup( tree.getData(),currentContext);
		popup.addReceiver(XappRootApplication.getRoot());
		
		return popup;
	}
	
	public XForm getFormView()
	{
		form = new XmlFormFactory(this,schema,viewController).createForm(tree.getData(),currentContext,new XappComponentStore());
		
		form.addReceiver(XappRootApplication.getRoot());
		
		return form;
	}
	
	public void processPopupAction(XmlElement action)
	{
		new XmlPopupFactory(this,schema).processAction(tree.getData(),currentContext,action);
		tree.refreshView();
		
		if(XmlApplication.autoSaveIsOn())
		{
			XmlApplication.save();
		}
	}

	public void dbChanged(XappXPathDB xPathDB) {
		
		viewController = new XmlSchemaViewController( viewController.getRoot(),viewDB.getRoot() );
		tree.setViewController( viewController );
		tree.refreshView();
		
		((XmlApplication)XappRootApplication.getRoot()).openForm(XappRootApplication.getConfig().getRoot(),currentContext);
	}
}
