package org.one.stone.soup.xapp.plugins;

import org.one.stone.soup.stringhelper.StringArrayHelper;
import org.one.stone.soup.xapp.XappPlugin;
import org.one.stone.soup.xapp.XappRootApplication;
import org.one.stone.soup.xapp.controller.xml.XmlApplication;
import org.one.stone.soup.xapp.factories.XappMenuFactory;
import org.one.stone.soup.xapp.factories.XappPluginFactory;
import org.one.stone.soup.xml.XmlElement;
import org.one.stone.soup.xml.helper.XmlPathSearch;

public class XApplicationSettings extends XappPlugin {
	
	public XApplicationSettings(String pluginFileName,XmlElement definition)
	{
		super(pluginFileName,definition);
		
		XmlPathSearch search = new XmlPathSearch(getDefinition());
		XmlElement xForm = null;
		if(XappRootApplication.getRoot() instanceof XmlApplication)
		{
			xForm = search.findElement("plugin/form[@name='Xml Application Settings Form']");			
		}
		else
		{
			xForm = search.findElement("plugin/form[@name='Application Settings Form']");
		}
		buildForm(xForm);	
	}
	
	public void initialise() {
	}

	public XmlElement getMenu() {
		XmlElement xMenu = XappMenuFactory.getMenuDefinition("Settings");
		XappMenuFactory.addMenuItem(xMenu,"Application Settings","showSettings");
		
		return xMenu;
	}

	public XmlElement getActions() {
		XmlElement xActions = XappPluginFactory.getActionsDefinition();
		XappPluginFactory.addAction(this,xActions,"showSettings");
		XappPluginFactory.addAction(this,xActions,"applySettings");
		return xActions;
	}

	public XmlElement getData()
	{
		updateFileBrowser();
		updateWindowSize();
		updateSchema();
		return form.getData();
	}
	
	public void setData(XmlElement data) {
		form.setData(data);
		applySettings();
	}
	
	private void updateFileBrowser()
	{
		StringBuffer data = new StringBuffer();
		XmlElement xDirectories = XappRootApplication.getComponentFactory().buildFileBrowser(null,null,null).getData();
		for(int loop=0;loop<xDirectories.getElementCount();loop++)
		{
			XmlElement item = xDirectories.getElementByIndex(loop);
			data.append( item.getAttributeValueByName("key") );
			data.append('\t');
			data.append( item.getValue() );
			data.append('\n');
		}

		XmlElement fileBrowser = new XmlElement("fileBrowser");
		fileBrowser.addValue( data.toString() );
		XmlElement xData = new XmlElement("data");
		xData.addChild(fileBrowser);
		form.setData(xData);

	}
	
	private void updateWindowSize()
	{
		XmlElement width = new XmlElement("width");
		width.addValue( ""+XappRootApplication.getMainWidth() );
		
		XmlElement height = new XmlElement("height");
		height.addValue( ""+XappRootApplication.getMainHeight() );

		XmlElement xData = new XmlElement("data");
		xData.addChild(width);
		xData.addChild(height);
		
		form.setData(xData);

	}	
	
	private void updateSchema()
	{
		if(XappRootApplication.getRoot() instanceof XmlApplication)
		{
			String schemaFile = ((XmlApplication)XappRootApplication.getRoot()).getSchemaFile();		
			XmlElement xData = new XmlElement("data");
			xData.addChild("schema").addValue(schemaFile);
		
			form.setData(xData);
		}

	}		
	
	public void showSettings()
	{	
		updateSchema();
		updateFileBrowser();
		updateWindowSize();
		frame.show();
	}
	
	public void applySettings()
	{
		XmlElement data = form.getData();
		
		int width = Integer.parseInt( data.getElementByName("width").getValue() );
		int height = Integer.parseInt( data.getElementByName("height").getValue() );
		boolean autosave = Boolean.valueOf( data.getElementByName("autosave").getValue() ).booleanValue();
		String directories = data.getElementByName("fileBrowser").getValue();
		
		XappRootApplication.setAutosave(autosave);
		XappRootApplication.setMainWidth(width);
		XappRootApplication.setMainHeight(height);
		
		String[] entries = StringArrayHelper.parseFields( directories,'\n' );
		XmlElement xDirectories = new XmlElement("xDirectories");
		for(int loop=0;loop<entries.length;loop++)
		{
			String[] args = StringArrayHelper.parseFields(entries[loop],'\t');
			XmlElement item = xDirectories.addChild( "item" );
			item.addAttribute( "key",args[0] );
			item.addValue( args[1] );
		}
		XappRootApplication.getComponentFactory().buildFileBrowser(null,null,null).setData(xDirectories);
		
		XmlElement root = data.getElementByName("root");
		if(root!=null && root.getValue().length()>0)
		{
			System.setProperty("user.dir",root.getValue());
		}
		
		if(XappRootApplication.getRoot() instanceof XmlApplication)
		{
			String schema = data.getElementByName("schema").getValue();
			if(schema.length()>0)
			{
				((XmlApplication)XappRootApplication.getRoot()).openSchema( schema,false );
			}
		}
		
		frame.hide();
	}
}
