package org.one.stone.soup.xapp.swing.components;

import java.awt.Component;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JTree;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import org.one.stone.soup.browser.Browser;
import org.one.stone.soup.grfx.ImageFactory;
import org.one.stone.soup.xapp.XappPersister;
import org.one.stone.soup.xapp.XappReceiver;
import org.one.stone.soup.xapp.XappSender;
import org.one.stone.soup.xapp.components.XappComponent;
import org.one.stone.soup.xapp.components.tree.XTree;
import org.one.stone.soup.xapp.controller.xml.XmlSchemaViewController;
import org.one.stone.soup.xml.XmlElement;
import org.one.stone.soup.xml.helper.XmlHelper;

public class XSwingTree extends JTree implements XappSender,XappPersister, XTree{

	private XmlSchemaViewController viewController;
	private XappReceiver receiver;
	private XmlElement data;
	private Hashtable nodes = new Hashtable();
	
	private class XTreeNode implements TreeNode
	{
		private XTreeNode parent;
		private XmlElement xData;
		private Vector children = new Vector();
		
		public XTreeNode(XTreeNode parent,XmlElement data)
		{
			nodes.put(data,this);
			this.parent = parent;
			xData = data;
			buildNode();
		}
		
		public TreePath getTreePath()
		{
			Vector path = new Vector();
			path.add(this);
			
			if(parent!=null)
			{
				parent.getTreePath(path);
			}
			
			XTreeNode[] pathArray = new XTreeNode[ path.size() ];
			for(int loop=0;loop<path.size();loop++)
			{
				pathArray[loop]=(XTreeNode)path.elementAt(loop);
			}
			
			return new TreePath(pathArray);
		}
		
		private void getTreePath(Vector path)
		{
			path.add(0,this);
			if(parent!=null)
			{
				parent.getTreePath(path);
			}			
		}
		
		public XmlElement getData()
		{
			return xData;
		}
		
		private void buildNode()
		{
			for(int loop=0;loop<xData.getElementCount();loop++)
			{
				XmlElement xChild = xData.getElementByIndex(loop);
				if(xChild!=null)
				{
					children.addElement( new XTreeNode(this,xChild) );
				}
			}
		}
		
		public Enumeration children() {
			return children.elements();
		}

		public boolean getAllowsChildren() {
			return !isLeaf();
		}

		public TreeNode getChildAt(int childIndex)
		{
			if(childIndex>-1 && childIndex<children.size())
			{
				return (TreeNode)children.elementAt(childIndex);
			}
			else
			{
				return null;
			}
		}

		public int getChildCount() {
			return children.size();
		}

		public int getIndex(TreeNode node) {
			return children.indexOf(node);
		}

		public TreeNode getParent() {
			return parent;
		}

		public boolean isLeaf() {
			if(xData.getElementCount()==0)
			{
				return true;
			}
			else
			{
				return false;
			}
		}
		
		public String toString()
		{
			return viewController.getNameForNode(xData);
		}
	}

	private class XTreeModel extends DefaultTreeModel
	{
		private XTreeModel(XmlElement xData)
		{
			super( new XTreeNode(null,xData) );
		}
	}
	
	private class XTreeCellRenderer extends DefaultTreeCellRenderer
	{
	    public Component getTreeCellRendererComponent(
	    		  JTree tree,
	    		  Object value,
				  boolean sel,
				  boolean expanded,
				  boolean leaf, int row,
				  boolean hasFocus
		)
	    {
	    	DefaultTreeCellRenderer node = (DefaultTreeCellRenderer)super.getTreeCellRendererComponent(tree,value,sel,expanded,leaf,row,hasFocus);
	    	
	    	if(!(value instanceof XTreeNode))
	    	{
	    		return node;
	    	}
	    	
	    	XmlElement xNode = ((XTreeNode)value).getData();
	    	
	    	String iconFileName = viewController.getIconFileNameForNode(xNode,expanded);
	    	if(iconFileName!=null)
	    	{
	    		Image image = (Image)Browser.getResource( iconFileName );
	    		if(image==null)
	    		{
	    			image = ImageFactory.loadImage(iconFileName);
	    			Browser.storeResource(iconFileName,image);
	    		}
	    		node.setIcon( new ImageIcon( image ) );
	    	}
	    	
	    	return node;
	    }
	}
	
	public XSwingTree(XmlElement xData,XmlSchemaViewController viewController)
	{
		super();

		this.viewController = viewController;

		buildTree(xData);
		
	    addMouseListener (
	            new MouseAdapter () {
	               public void mouseReleased( MouseEvent e ) {
	            	   
	            	   TreePath path = getPathForLocation( e.getX(),e.getY() );
	            	   if(path==null)
	            	   {
	            		   return;
	            	   }
	            	   XTreeNode node = (XTreeNode)path.getLastPathComponent();
	            	   
	                  if ( e.getButton()==MouseEvent.BUTTON1)
	                  {
	                      selectionMade( node,e.getX(),e.getY(),false );
	                  }
	                  else
	                  {
	                	  setSelectionPath( path );
	                      selectionMade( node,e.getX(),e.getY(),true );
	                  }
	                 } 
	               }
	            );
	}
	
	/* (non-Javadoc)
	 * @see org.one.stone.soup.xapp.components.tree.XTree#buildTree(org.one.stone.soup.xml.XmlElement)
	 */
	public void buildTree(XmlElement xData)
	{
		setName( xData.getAttributeValueByName("name") );
		
		this.setCellRenderer( new XTreeCellRenderer() );
	    
	    setData(xData);
	}

	/* (non-Javadoc)
	 * @see org.one.stone.soup.xapp.components.tree.XTree#setData(org.one.stone.soup.xml.XmlElement)
	 */
	public void setData(XmlElement xData)
	{
		this.data = xData;
		refreshView();
	}

	/* (non-Javadoc)
	 * @see org.one.stone.soup.xapp.components.tree.XTree#setViewController(org.one.stone.soup.xapp.controller.xml.XmlSchemaViewController)
	 */
	public void setViewController(XmlSchemaViewController viewController)
	{
		this.viewController = viewController;
	}
	
	/* (non-Javadoc)
	 * @see org.one.stone.soup.xapp.components.tree.XTree#refreshView()
	 */
	public void refreshView()
	{
		XmlElement xSelection = null;
		if(this.getSelectionPath()!=null)
		{
			xSelection = ((XTreeNode)this.getSelectionPath().getLastPathComponent()).getData();
			
			if( XmlHelper.getParent(data,xSelection)==null && this.getSelectionPath().getPathCount()>=2) // Handle deletions
			{
				xSelection = ((XTreeNode)this.getSelectionPath().getPathComponent(this.getSelectionPath().getPathCount()-2)).getData();
			}
		}
		
		nodes = new Hashtable();
		this.setModel( new XTreeModel(data) );
		this.invalidate();
		
		if(xSelection!=null)
		{
			XTreeNode selection = (XTreeNode)nodes.get(xSelection);
			
			if(selection!=null)
			{
				setSelectionPath( selection.getTreePath() );
				expandPath( selection.getTreePath() );
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see org.one.stone.soup.xapp.components.tree.XTree#getElementAt(int, int)
	 */
	public XmlElement getElementAt(int x,int y)
	{
		TreePath path = getPathForLocation(x,y);
		if(path==null)
		{
			return null;
		}
		return ((XTreeNode)path.getLastPathComponent()).getData();
	}
	
	private void selectionMade(XTreeNode node,int x,int y,boolean rightButton)
	{
		XmlElement action = new XmlElement("treeAction");
		if(rightButton==true)
		{
			action.addAttribute("button","right");
		}
		else
		{
			action.addAttribute("button","left");
		}

		action.addAttribute("x",""+x);
		action.addAttribute("y",""+y);
		action.addChild( data );
		action.addChild( node.getData() );
		
		//System.out.println( action.toXml() );
		if(receiver!=null)
		{
			receiver.receiveXappData( action );
		}
	}

	/* (non-Javadoc)
	 * @see org.one.stone.soup.xapp.components.tree.XTree#addReceiver(org.one.stone.soup.xapp.XappReceiver)
	 */
	public void addReceiver(XappReceiver receiver) {
		this.receiver = receiver;
	}

	/* (non-Javadoc)
	 * @see org.one.stone.soup.xapp.components.tree.XTree#removeReceiver(org.one.stone.soup.xapp.XappReceiver)
	 */
	public void removeReceiver(XappReceiver receiver) {
		receiver=null;
	}
	
	/* (non-Javadoc)
	 * @see org.one.stone.soup.xapp.components.tree.XTree#getData()
	 */
	public XmlElement getData()
	{
		return data;
	}
	
	/* (non-Javadoc)
	 * @see org.one.stone.soup.xapp.components.tree.XTree#selectNode(org.one.stone.soup.xml.XmlElement)
	 */
	public void selectNode(XmlElement data)
	{
		XTreeNode node = (XTreeNode)nodes.get(data);
		
		setSelectionPath(node.getTreePath());
	}
	
	/* (non-Javadoc)
	 * @see org.one.stone.soup.xapp.components.tree.XTree#selectNodes(org.one.stone.soup.xml.XmlElement)
	 */
	public void selectNodes(XmlElement data)
	{
		TreePath[] paths = new TreePath[data.getElementCount()];
		
		for(int loop=0;loop<data.getElementCount();loop++)
		{
			XTreeNode node = (XTreeNode)nodes.get(data.getElementByIndex(loop));
			paths[loop]=node.getTreePath();
		}
		
		setSelectionPaths( paths );
	}

	public void add(String name, XappComponent component) {
		// TODO Auto-generated method stub
		
	}

	public void add(XappComponent component) {
		// TODO Auto-generated method stub
		
	}

	public void refresh() {
		refreshView();
	}

	public void actionPerformed(ActionEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void addActionListener(ActionListener actionListener) {
		// TODO Auto-generated method stub
		
	}
	

	public void setTip(String text)
	{
		setToolTipText(text);
	}
}
